@@ -100,7 +100,9 @@ class DeformableAttention1D(nn.Module):
         offset_scale = None,
         offset_groups = None,
         offset_kernel_size = 6,
-        cpb_log_distance = True
+        cpb_log_distance = True,
+        group_queries = True,
+        group_key_values = True
     ):
         super().__init__()
         offset_scale = default(offset_scale, downsample_factor)
@@ -131,8 +133,9 @@ class DeformableAttention1D(nn.Module):
         self.rel_pos_bias = CPB(dim // 4, offset_groups = offset_groups, heads = heads, depth = 2, log_distance = cpb_log_distance)
 
         self.dropout = nn.Dropout(dropout)
-        self.to_q = nn.Conv1d(dim, inner_dim, 1, groups = offset_groups, bias = False)
-        self.to_kv = nn.Conv1d(dim, inner_dim * 2, 1, groups = offset_groups, bias = False)
+        self.to_q = nn.Conv1d(dim, inner_dim, 1, groups = offset_groups if group_queries else 1, bias = False)
+        self.to_k = nn.Conv1d(dim, inner_dim, 1, groups = offset_groups if group_key_values else 1, bias = False)
+        self.to_v = nn.Conv1d(dim, inner_dim, 1, groups = offset_groups if group_key_values else 1, bias = False)
         self.to_out = nn.Conv1d(inner_dim, dim, 1)
 
     def forward(self, x, return_vgrid = False):
@@ -172,7 +175,7 @@ class DeformableAttention1D(nn.Module):
 
         # derive key / values
 
-        k, v = self.to_kv(kv_feats).chunk(2, dim = 1)
+        k, v = self.to_k(kv_feats), self.to_v(kv_feats)
 
         # scale queries
 
