@@ -106,7 +106,9 @@ class DeformableAttention3D(nn.Module):
         downsample_factor = 4,
         offset_scale = None,
         offset_groups = None,
-        offset_kernel_size = 6
+        offset_kernel_size = 6,
+        group_queries = True,
+        group_key_values = True
     ):
         super().__init__()
         downsample_factor = cast_tuple(downsample_factor, length = 3)
@@ -138,8 +140,9 @@ class DeformableAttention3D(nn.Module):
         self.rel_pos_bias = CPB(dim // 4, offset_groups = offset_groups, heads = heads, depth = 2)
 
         self.dropout = nn.Dropout(dropout)
-        self.to_q = nn.Conv3d(dim, inner_dim, 1, groups = offset_groups, bias = False)
-        self.to_kv = nn.Conv3d(dim, inner_dim * 2, 1, groups = offset_groups, bias = False)
+        self.to_q = nn.Conv3d(dim, inner_dim, 1, groups = offset_groups if group_queries else 1, bias = False)
+        self.to_k = nn.Conv3d(dim, inner_dim, 1, groups = offset_groups if group_key_values else 1, bias = False)
+        self.to_v = nn.Conv3d(dim, inner_dim, 1, groups = offset_groups if group_key_values else 1, bias = False)
         self.to_out = nn.Conv3d(inner_dim, dim, 1)
 
     def forward(self, x, return_vgrid = False):
@@ -182,7 +185,7 @@ class DeformableAttention3D(nn.Module):
 
         # derive key / values
 
-        k, v = self.to_kv(kv_feats).chunk(2, dim = 1)
+        k, v = self.to_k(kv_feats), self.to_v(kv_feats)
 
         # scale queries
 
